package cn.fw.rp.common.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.UnsupportedEncodingException;
import java.lang.reflect.Array;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created with IntelliJ IDEA
 * Created By Devin
 * Date: 2018/4/3
 * Time: 17:22
 * Description: 通用工具类
 */
public class PublicUtil {
    public static final String DATA_FORMAT = "yyyy-MM-dd";
    public static final String DATA_FORMAT_CLEAR = "yyyyMMdd";
    public static final String TIME_FORMAT = "yyyy-MM-dd HH:mm:ss";
    public static final String TIME_FORMAT_S = "yyyy-MM-dd HH:mm:ss.sss";
    protected static Logger logger = LoggerFactory.getLogger(PublicUtil.class);

    public PublicUtil() {
    }

    public static String toShowHtml(String str) {
        if (str == null) {
            return null;
        } else {
            String html = new String(str);
            html = replace(html, "&amp;", "&");
            html = replace(html, "&lt;", "<");
            html = replace(html, "&gt;", ">");
            html = replace(html, "&quot;", "\"");
            html = replace(html, "<br>", "\n");
            return html;
        }
    }

    public static Boolean containsStr(String source, String str) {
        return Boolean.valueOf(isNotEmpty(source) && source.contains(str));
    }

    public static int getLength(String text) {
        int textLength = text.length();
        int length = textLength;

        for (int i = 0; i < textLength; ++i) {
            if (String.valueOf(text.charAt(i)).getBytes().length > 1) {
                ++length;
            }
        }

        return length % 2 == 0 ? length / 2 : length / 2 + 1;
    }

    public static String subMaxString(String str, Integer length) {
        if (isNotEmpty(str) && str.length() > length.intValue()) {
            Integer realLength = Integer.valueOf(0);
            Integer len = Integer.valueOf(str.length());

            for (int i = 0; i < len.intValue(); ++i) {
                if (String.valueOf(str.charAt(i)).getBytes().length > 1) {
                    realLength = Integer.valueOf(realLength.intValue() + 2);
                } else {
                    realLength = Integer.valueOf(realLength.intValue() + 1);
                }

                if (realLength.intValue() > length.intValue()) {
                    str = toAppendStr(new Object[]{replaceBlank(str).substring(0, i), "..."});
                    break;
                }
            }
        }

        return str;
    }

    public static String replaceBlank(String str) {
        String dest = "";
        if (isNotEmpty(str)) {
            Pattern p = Pattern.compile("\\s*|\t|\r|\n");
            Matcher m = p.matcher(str);
            dest = m.replaceAll("");
        }

        return dest;
    }

    public static String replace(String source, String oldString, String newString) {
        StringBuffer output = new StringBuffer();
        if (!source.equals("") && source != null) {
            int lengthOfSource = source.length();
            int lengthOfOld = oldString.length();

            int posStart;
            int pos;
            for (posStart = 0; (pos = source.indexOf(oldString, posStart)) >= 0; posStart = pos + lengthOfOld) {
                output.append(source.substring(posStart, pos));
                output.append(newString);
            }

            if (posStart < lengthOfSource) {
                output.append(source.substring(posStart));
            }

            return output.toString();
        } else {
            return "";
        }
    }

    public static String getAuthorizeURLBase(String url, String appId, String componentAppId) {
//        url = "https://open.weixin.qq.com/connect/oauth2/authorize?appid=" + appId
//                + "&redirect_uri=" + url
//                + "&response_type=code"
//                + "&scope=snsapi_userinfo"
//                + "&component_appid=" + componentAppId
//                + "&state=123#wechat_redirect";
        url = "https://open.weixin.qq.com/connect/oauth2/authorize?appid=" + appId
                + "&redirect_uri=" + url
                + "&response_type=code"
                + "&scope=snsapi_userinfo"
                + "&state=123"
                + "&component_appid=" + componentAppId
                + "&connect_redirect=1#wechat_redirect";
        return url;
    }

    public static String getUrlEncode(String url) {
        try {
            url = URLEncoder.encode(url, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return url;
    }

    public static String replaceAll(String str, String oldStr, String newStr) {
        int length = str.indexOf(oldStr);
        int j = 0;

        for (String temp = ""; length != -1; length = str.indexOf(oldStr)) {
            ++j;
            if (j == 10) {
                break;
            }

            temp = str.substring(length + 1);
            str = str.substring(0, length);
            str = str + newStr + temp;
        }

        return str;
    }

    public static int parseInt(Object value, Integer defaultValue) {
        try {
            return Integer.parseInt(String.valueOf(value));
        } catch (Exception var3) {
            return defaultValue.intValue();
        }
    }

    public static long parseLong(Object value, Long defaultValue) {
        try {
            return Long.parseLong(String.valueOf(value));
        } catch (Exception var3) {
            return defaultValue.longValue();
        }
    }

    public static Date parseDate(Object date) {
        return parseDate(String.valueOf(date));
    }

    public static Date parseDate(String date) {
        try {
            SimpleDateFormat ex = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            return ex.parse(date);
        } catch (Exception var2) {
            return null;
        }
    }

    public static Date parseDate(String date, String format) {
        try {
            SimpleDateFormat ex = new SimpleDateFormat(format);
            return ex.parse(date);
        } catch (Exception var3) {
            return null;
        }
    }

    public static String parseString(Object value, String defaultValue) {
        return null != value && !"null".equals(String.valueOf(value).toLowerCase()) ? String.valueOf(value) : defaultValue;
    }

    public static String getCurrentTime() {
        return getCurrentTime("yyyy-MM-dd HH:mm:ss");
    }

    public static Date getCurrentDate() {
        return new Date();
    }

    public static String getCurrentTime(String format) {
        Date nowTime = new Date();
        SimpleDateFormat fmt = new SimpleDateFormat(format);
        return fmt.format(nowTime);
    }

    public static String getMySQLDateTimeFormat() {
        return "yyyy-MM-dd HH:mm:ss";
    }

    public static String fmtDate(Date date, String fmt) {
        if (null == date) {
            return "";
        } else {
            SimpleDateFormat f = new SimpleDateFormat(fmt);
            return f.format(date);
        }
    }

    public static String fmtDate(Date date) {
        if (null == date) {
            return "";
        } else {
            SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            return f.format(date);
        }
    }

    /**
     * 转换为时间（天,时:分:秒.毫秒）
     *
     * @param timeMillis
     * @return
     */
    public static String formatDateTime(long timeMillis) {
        long day = timeMillis / (24 * 60 * 60 * 1000);
        long hour = (timeMillis / (60 * 60 * 1000) - day * 24);
        long min = ((timeMillis / (60 * 1000)) - day * 24 * 60 - hour * 60);
        long s = (timeMillis / 1000 - day * 24 * 60 * 60 - hour * 60 * 60 - min * 60);
        long sss = (timeMillis - day * 24 * 60 * 60 * 1000 - hour * 60 * 60 * 1000 - min * 60 * 1000 - s * 1000);
        return (day > 0 ? day + "," : "") + hour + ":" + min + ":" + s + "." + sss;
    }

    public static String nextId() {
        return UUID.randomUUID().toString().replaceAll("-", "");
    }

    public static String getUUID() {
        String uid = "0";
        SimpleDateFormat sf = new SimpleDateFormat("yyyyMMddHHmmssSSS");
        String tempId = sf.format(new Date());
        if (Long.parseLong(uid) >= Long.parseLong(tempId)) {
            uid = Long.parseLong(uid) + 1L + "";
        } else {
            uid = tempId;
        }

        return uid + getRandomString(10);
    }

    public static String getRandomString(int size) {
        char[] c = new char[]{'1', '2', '3', '4', '5', '6', '7', '8', '9', '0', 'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', 'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'z', 'x', 'c', 'v', 'b', 'n', 'm'};
        Random random = new Random();
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < size; ++i) {
            sb.append(c[Math.abs(random.nextInt()) % c.length]);
        }

        return sb.toString();
    }

    public static String getRandomNumber(int size) {
        char[] c = new char[]{'1', '2', '3', '4', '5', '6', '7', '8', '9', '0'};
        Random random = new Random();
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < size; ++i) {
            sb.append(c[Math.abs(random.nextInt()) % c.length]);
        }

        return sb.toString();
    }

    public static boolean isNotEmpty(Object obj) {
        return !isEmpty(obj);
    }

    public static boolean isEmpty(Object obj) {
        if (obj == null) {
            return true;
        } else if (obj instanceof String) {
            return obj.equals("");
        } else if (obj instanceof Collection) {
            Collection map1 = (Collection) obj;
            return map1.size() == 0;
        } else if (obj instanceof Map) {
            Map map = (Map) obj;
            return map.size() == 0;
        } else {
            return obj.getClass().isArray() ? Array.getLength(obj) == 0 : false;
        }
    }

    // 获取随机字符串
    public static String getNonceStr() {
        String chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        String res = "";
        for (int i = 0; i < 16; i++) {
            Random rd = new Random();
            res += chars.charAt(rd.nextInt(chars.length() - 1));
        }
        return res;
    }

    public static boolean isDigitalString(String str) {
        if (isEmpty(str)) {
            return false;
        } else {
            try {
                Double.parseDouble(str);
                return true;
            } catch (Exception var2) {
                return false;
            }
        }
    }

    public static boolean equalsFlag(Object str) {
        try {
            if (str != null && "true".equals(str)) {
                return true;
            }
        } catch (Exception var2) {
            var2.printStackTrace();
        }

        return false;
    }

    public static boolean isNumeric(String str) {
        try {
            return str != null && str.matches("\\d*.?\\d*");
        } catch (Exception var2) {
            var2.printStackTrace();
            return false;
        }
    }

    public static int arrayIndexOf(Object array, Object value) {
        if (array != null && value != null) {
            int len = Array.getLength(array);

            for (int i = 0; i < len; ++i) {
                if (value.equals(Array.get(array, i))) {
                    return i;
                }
            }

            return -1;
        } else {
            return -1;
        }
    }

    public static String toUtf8String(String s) {
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < s.length(); ++i) {
            char c = s.charAt(i);
            if (c >= 0 && c <= 255) {
                sb.append(c);
            } else {
                byte[] b;
                try {
                    b = Character.toString(c).getBytes("utf-8");
                } catch (Exception var7) {
                    var7.printStackTrace();
                    b = new byte[0];
                }

                for (int j = 0; j < b.length; ++j) {
                    int k = b[j];
                    if (k < 0) {
                        k += 256;
                    }

                    sb.append("%" + Integer.toHexString(k).toUpperCase());
                }
            }
        }

        return sb.toString();
    }

    public static String sicenToComm(Object value) {
        return value == null ? null : (value instanceof Double ? sicenToCommDouble(((Double) value).doubleValue()) : String.valueOf(value));
    }

    public static String sicenToCommDouble(double value) {
        String retValue = null;
        DecimalFormat df = new DecimalFormat();
        df.setMinimumFractionDigits(0);
        df.setMaximumFractionDigits(5);
        retValue = df.format(value);
        retValue = retValue.replaceAll(",", "");
        return retValue;
    }

    public static String roundStr(Double value) {
        DecimalFormat df = new DecimalFormat("#.00");
        return df.format(value);
    }

    public static Double round(Double value, int count) {
        try {
            BigDecimal e = new BigDecimal(value.doubleValue());
            return Double.valueOf(e.setScale(count, 4).doubleValue());
        } catch (Exception var3) {
            logger.warn("round double" + value);
            return Double.valueOf(0.0D);
        }
    }

    public static Double round(Object value, int count) {
        return round(parseDouble(value), count);
    }

    public static Double parseDouble(Object value) {
        return value == null ? Double.valueOf(0.0D) : parseDouble(String.valueOf(value));
    }

    public static Double parseDouble(String value) {
        return parseDouble(value, Double.valueOf(0.0D));
    }

    public static Double parseDouble(String value, Double dafualtVal) {
        Double r = dafualtVal;

        try {
            if (value != null) {
                value = value.trim();
                if (value.endsWith("%")) {
                    value = value.substring(0, value.length() - 1);
                }

                r = Double.valueOf(Double.parseDouble(value));
            }
        } catch (Exception var4) {
            logger.warn("转换失败：" + var4.getMessage());
        }

        return r;
    }

    public static String toLowerCaseFirstOne(String s) {
        return Character.isLowerCase(s.charAt(0)) ? s : Character.toLowerCase(s.charAt(0)) + s.substring(1);
    }

    public static String toUpperCaseFirstOne(String s) {
        return Character.isUpperCase(s.charAt(0)) ? s : Character.toUpperCase(s.charAt(0)) + s.substring(1);
    }

    public static String toAppendStr(Object... strs) {
        StringBuffer sb = new StringBuffer();
        Object[] var2 = strs;
        int var3 = strs.length;

        for (int var4 = 0; var4 < var3; ++var4) {
            Object str = var2[var4];
            if (isNotEmpty(str)) {
                sb.append(str);
            }
        }

        return sb.toString();
    }

    public static boolean match(String pattern, String str) {
        Pattern p = Pattern.compile(pattern);
        Matcher m = p.matcher(str);
        return m.find();
    }

    public static String toStrString(Object obj) {
        return isNotEmpty(obj) ? String.valueOf(obj) : "";
    }

    public static String toStrStringNull(Object obj) {
        return isNotEmpty(obj) ? String.valueOf(obj) : null;
    }

    public static Object fmtDate(ZonedDateTime val, String format) {
        return val == null ? null : val.format(DateTimeFormatter.ofPattern(format));
    }

    public static Object fmtDate(ZonedDateTime val) {
        return fmtDate(val, "yyyy-MM-dd HH:mm:ss");
    }

    public static String getRandomBsString(int length) {
        String str = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        Random random = new Random();
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < length; i++) {
            int number = random.nextInt(62);
            sb.append(str.charAt(number));
        }
        return sb.toString();
    }

    /**
     * 将字符串进行sha1加密
     *
     * @param str 需要加密的字符串
     * @return 加密后的内容
     */
    public static String sha1(String str) {
        try {
            MessageDigest digest = MessageDigest.getInstance("SHA-1");
            digest.update(str.getBytes());
            byte messageDigest[] = digest.digest();
            // Create Hex String
            StringBuffer hexString = new StringBuffer();
            // 字节数组转换为 十六进制 数
            for (int i = 0; i < messageDigest.length; i++) {
                String shaHex = Integer.toHexString(messageDigest[i] & 0xFF);
                if (shaHex.length() < 2) {
                    hexString.append(0);
                }
                hexString.append(shaHex);
            }
            return hexString.toString();

        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        return "";
    }

    public static boolean isBlank(CharSequence cs) {
        int strLen;
        if (cs != null && (strLen = cs.length()) != 0) {
            for (int i = 0; i < strLen; ++i) {
                if (!Character.isWhitespace(cs.charAt(i))) {
                    return false;
                }
            }

            return true;
        } else {
            return true;
        }
    }

    public static boolean isNotBlank(CharSequence cs) {
        return !isBlank(cs);
    }

    /**
     * 替换掉HTML标签方法
     */
    public static String replaceHtml(String html) {
        if (isBlank(html)) {
            return "";
        }
        String regEx = "<.+?>";
        Pattern p = Pattern.compile(regEx);
        Matcher m = p.matcher(html);
        String s = m.replaceAll("");
        return s;
    }
}

