package cn.fw.rp.web.controller;

import cn.fw.data.base.domain.common.Message;
import cn.fw.rp.common.enums.FileTypeEnums;
import cn.fw.rp.common.util.DateUtils;
import cn.fw.rp.dao.FileListDao;
import cn.fw.rp.dao.ProblemFileDao;
import cn.fw.rp.db.FileList;
import cn.fw.rp.db.ProblemFile;
import net.coobird.thumbnailator.Thumbnails;
import org.apache.catalina.connector.ClientAbortException;
import org.apache.tomcat.util.http.fileupload.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.UUID;

/**
 * @author suchu
 * @since 2018/5/9 10:59
 */
@Controller
@RequestMapping("/file")
public class FileController {

    @Value("${upload.path}")
    String uploadPath;

    @Autowired
    FileListDao dao;

    @Autowired
    ProblemFileDao problemFileDao;

    public static void main(String[] args) throws IOException {
        Thumbnails.of("F:\\RP_UPLOAD\\2018\\05\\09\\1db414949be24c3d9de479d2cbfa4c81.jpg")
                .size(100, 100).toFile(new File("F:\\\\RP_UPLOAD\\test1.jpg"));
    }

    public String datePath() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd");
        return sdf.format(DateUtils.getNow());
    }

    @RequestMapping("/video/{fileId}")
    public void videoView(@PathVariable String fileId, HttpServletResponse response, HttpServletRequest request, @RequestHeader(value = "range", required = false) String range) {
        FileList fileList = dao.selectByFileId(fileId);
        File file = null;
        if (null != fileList) {
            file = new File(fileList.getFilePath());
            if (!file.exists()) {
                response.setStatus(204);//NO content
                return;
            }
        }

        //文件目录
//        File music = new File("E:\\CloudMusic\\林子祥 - 街头霸王榜.mp3");


        //开始下载位置
        long startByte = 0;
        //结束下载位置
        long endByte = file.length() - 1;

        //有range的话
        if (range != null && range.contains("bytes=") && range.contains("-")) {
            range = range.substring(range.lastIndexOf("=") + 1).trim();
            String ranges[] = range.split("-");
            try {
                //判断range的类型
                if (ranges.length == 1) {
                    //类型一：bytes=-2343
                    if (range.startsWith("-")) {
                        endByte = Long.parseLong(ranges[0]);
                    }
                    //类型二：bytes=2343-
                    else if (range.endsWith("-")) {
                        startByte = Long.parseLong(ranges[0]);
                    }
                }
                //类型三：bytes=22-2343
                else if (ranges.length == 2) {
                    startByte = Long.parseLong(ranges[0]);
                    endByte = Long.parseLong(ranges[1]);
                }

            } catch (NumberFormatException e) {
                startByte = 0;
                endByte = file.length() - 1;
            }
        }

        //要下载的长度（为啥要加一问小学数学老师去）
        long contentLength = endByte - startByte + 1;
        //文件名
        String fileName = file.getName();
        //文件类型
        String contentType = request.getServletContext().getMimeType(fileName);


        //各种响应头设置
        //参考资料：https://www.ibm.com/developerworks/cn/java/joy-down/index.html
        //坑爹地方一：看代码
        response.setHeader("Accept-Ranges", "bytes");
        //坑爹地方二：http状态码要为206
        response.setStatus(response.SC_PARTIAL_CONTENT);
        response.setContentType(contentType);
        response.setHeader("Content-Type", contentType);
        //这里文件名换你想要的，inline表示浏览器直接实用（我方便测试用的）
        //参考资料：http://hw1287789687.iteye.com/blog/2188500
        response.setHeader("Content-Disposition", "inline;filename=test.mp3");
        response.setHeader("Content-Length", String.valueOf(contentLength));
        //坑爹地方三：Content-Range，格式为
        // [要下载的开始位置]-[结束位置]/[文件总大小]
        response.setHeader("Content-Range", "bytes " + startByte + "-" + endByte + "/" + file.length());


        BufferedOutputStream outputStream = null;
        RandomAccessFile randomAccessFile = null;
        //已传送数据大小
        long transmitted = 0;
        try {
            randomAccessFile = new RandomAccessFile(file, "r");
            outputStream = new BufferedOutputStream(response.getOutputStream());
            byte[] buff = new byte[4096];
            int len = 0;
            randomAccessFile.seek(startByte);
            //坑爹地方四：判断是否到了最后不足4096（buff的length）个byte这个逻辑（(transmitted + len) <= contentLength）要放前面！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！！
            //不然会会先读取randomAccessFile，造成后面读取位置出错，找了一天才发现问题所在
            while ((transmitted + len) <= contentLength && (len = randomAccessFile.read(buff)) != -1) {
                outputStream.write(buff, 0, len);
                transmitted += len;
                //停一下，方便测试，用的时候删了就行了
                //   Thread.sleep(10);
            }
            //处理不足buff.length部分
            if (transmitted < contentLength) {
                len = randomAccessFile.read(buff, 0, (int) (contentLength - transmitted));
                outputStream.write(buff, 0, len);
                transmitted += len;
            }

            outputStream.flush();
            response.flushBuffer();
            randomAccessFile.close();
            System.out.println("下载完毕：" + startByte + "-" + endByte + "：" + transmitted);

        } catch (ClientAbortException e) {
            System.out.println("用户停止下载：" + startByte + "-" + endByte + "：" + transmitted);
            //捕获此异常表示拥护停止下载
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (randomAccessFile != null) {
                    randomAccessFile.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

    }

    @RequestMapping("/view/{fileId}")
    public void view(@PathVariable String fileId, HttpServletResponse response, HttpServletRequest request) throws IOException {

        FileList fileList = dao.selectByFileId(fileId);
        if (null != fileList) {
            File file = new File(fileList.getFilePath());
            if (!file.exists()) {
                response.setStatus(404);
                return;
            }
            FileInputStream fileInputStream = new FileInputStream(file);
            response.setContentType(fileList.getContentType());
            response.setContentLength(fileInputStream.available());
            byte buffer[] = new byte[4096];
            ServletOutputStream outputStream = response.getOutputStream();
            int length = fileInputStream.read(buffer);
            while (length != -1) {
                outputStream.write(buffer, 0, length);
                length = fileInputStream.read(buffer);
            }
            outputStream.close();
        } else {
            response.setStatus(404);
            System.out.println("无法下载...");
        }
    }

    public int getFlag(String contentType) {
        if (contentType.contains("image")) {
            return FileTypeEnums.IMAGE.getType();
        } else if (contentType.contains("video"))
            return FileTypeEnums.VIDEO.getType();
        else return FileTypeEnums.FILE.getType();
    }

    @RequestMapping("/image/thumb")
    public void thumbnails(HttpServletResponse response, @RequestParam String fileId, @RequestParam(defaultValue = "200") Integer width, @RequestParam(defaultValue = "200") Integer height) throws IOException {
        FileList fileList = dao.selectByFileId(fileId);
        if (width > 2000)
            width = 150;
        if (height > 2000) {
            height = 150;
        }
        if (null != fileList) {
            File file = new File(fileList.getFilePath());
            if (!file.exists()) {
                response.setStatus(404);
            } else {
                response.setContentType(fileList.getContentType());
                ServletOutputStream outputStream = response.getOutputStream();
                Thumbnails.of(file).size(width, height).toOutputStream(outputStream);
                outputStream.close();
            }
        } else {
            response.setStatus(404);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @RequestMapping("/relate")
    @ResponseBody
    public Message relate(String fileId, String problemId) {
        Assert.notNull(fileId, "file id must not be null");
        Assert.notNull(problemId, "problem id must not be null");
        ProblemFile problemFile = new ProblemFile();
        problemFile.setFileId(fileId);
        problemFile.setProblemId(problemId);
        problemFileDao.insert(problemFile);
        return Message.success("关联成功");
    }

    @Transactional(rollbackFor = Exception.class)
    @PostMapping("/upload")
    @ResponseBody
    public Message upload(@RequestParam("file") MultipartFile file, @RequestParam(name = "flag", defaultValue = "3") Integer flag, HttpServletRequest request) throws IOException {
        String contentType = file.getContentType();
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        Long fileLength = file.getSize();
        //int flag = getFlag(contentType);
        String fileName = file.getOriginalFilename();
        String fileSuffix = fileName.substring(fileName.lastIndexOf("."));
        FileList fileList = new FileList();
        fileList.setContentType(contentType);
        fileList.setFileId(uuid);
        fileList.setUploadTime(DateUtils.getNow());
        fileList.setFileName(fileName);
        fileList.setFileLength(fileLength);
        fileList.setFlag(flag);
        // IOUtils.
        File rootPath = new File(uploadPath);
        if (!rootPath.exists()) {
            rootPath.mkdir();
        }
        String datePath = datePath();

        File outPath = new File(rootPath, datePath);
        if (!outPath.exists()) {
            outPath.mkdirs();
        }
        File outFile = new File(outPath, uuid + fileSuffix);
        fileList.setFilePath(outFile.getAbsolutePath());
        FileOutputStream fos = new FileOutputStream(outFile);
        IOUtils.copy(file.getInputStream(), fos);
        fos.flush();
        fos.close();
        dao.insert(fileList);
        Message message = Message.success();
        message.setData(fileList);
        return message;

    }
}
